/*
 * keys.c
 *
 * Tastenabfrage
 *
 * Created: 16.08.2013
 *  Author: Nicolas (www.DL1DOW.de)
 */  

#include <avr/io.h>
#include "keys.h"     // Schalter, Taster, Potis
#include "ioMapping.h"


/* Taster */
void key_init(void) {
   TAST_DDR  &= ~(1<<TAST_P);
   TAST_PORT |=  (1<<TAST_P); 
   }


// Taster abfragen und entprellen (active Low), genau eine Ausgabe pro Tastendruck
// zum Pollen in der Hauptschleife
char getkeystroke(void) {
   static char inpstat = 0x00; // Status Pin
   static char oldstat = 0x00; // Letze Ausgabe
   static char newstat = 2;
   char out;

   char temp = (TAST_PIN>>TAST_P) & 0x01;
   inpstat = (inpstat<<1) | temp;
   if (inpstat == 0x00) {
     newstat = 1;     
	 }
   else if (inpstat == 0xFF) {
     newstat = 0;
	 }
   if ((newstat == 1) && (oldstat == 0)) 
     out = 1;
   else
	 out = 0;

   oldstat = newstat;
   return out;
}



// Taster abfragen und entprellen (active Low), Dauer
volatile uint16_t sgl_time = 0;
void key_poll(void) {
  static char inpstat = 0x00; // Status Pin
  static char oldstat = 0x00; // Letze Ausgabe
  static char newstat = 2;
  static uint16_t ltime = 0;

  char temp = (TAST_PIN>>TAST_P) & 0x01;
  inpstat = (inpstat<<1) | temp;
  if (inpstat == 0x00) {
    newstat = 1;
  }
  else if (inpstat == 0xFF) {
    newstat = 0;
  }

  if (newstat == 1) ltime++;
  
  // fallende Flanke  
  if ((newstat == 0) && (oldstat == 1)) {
    sgl_time = ltime;
    ltime = 0;
  }  
  
  oldstat = newstat;
}


uint16_t key_time(void) {
  uint16_t temp = sgl_time;
  sgl_time = 0;
  return temp;
}  