/*
 * led_multiplex.c
 *
 * Created: 10.08.2013 18:00:51
 *  Author: Nicolas (www.DL1DOW.de)
 */ 

#include <avr/io.h>
#include <stdlib.h> // abs
#include "led_multiplex.h"


#define LED_BLINK_DELAY 100 
// uint8_t !



char led_lut0[] = {0xfa,0x18,0xec,0x7c,0x1e,0x76,0xf6,0x38,0xfe,0x7e,0xbe,0xd6,0xe2,0xdc,0xe6,0xa6,0x00}; // Zifferndarstellung
static char sgl_digits[N_DIGITS]; // Ziffernpuffer
static uint8_t sgl_isblink;

// Multiplexen der LED-Anzeige; fuer schnelle Schleife gedacht
void led_mpx_multiplex (void) {
  static uint8_t i = 0;
  static uint8_t waitblink = 0;
  waitblink++;
  i++;
  if (i==N_DIGITS) {i=0;}


  uint8_t digit = sgl_digits[i];  
  
  // alles aus (active low)
  LED_PORT = 0xFF;
  GRP_PORT |= (1<<GRP_0);
  GRP_PORT |= (1<<GRP_1);
  GRP_PORT |= (1<<GRP_2);
  
  if ((waitblink < LED_BLINK_DELAY) && sgl_isblink )  return;
  if (waitblink > 2*LED_BLINK_DELAY) waitblink = 0;

  // Ziffer darstellen
  switch (i) {
    case 0:
      GRP_PORT &= ~(1<<GRP_0);
      break;
    case 1:
      GRP_PORT &= ~(1<<GRP_1);
      break;
    case 2:
      GRP_PORT &= ~(1<<GRP_2);
    }
    LED_PORT = ~digit;
}



// Zahl auf LED darstellen; 
void led_mpx_putnum(int16_t num) {
  uint8_t digits[N_DIGITS];
  
  num = abs(num);
  
  // auf darstellbaren Bereich beschraenken
  switch(N_DIGITS) {
    case 1: num %=10;    break;
    case 2: num %=100;   break;
    case 3: num %=1000;  break;
    case 4: num %=10000; break;
  }

  // Ziffen erzeugen und zwischenpuffern
  for (uint8_t i = 0; i<N_DIGITS;i++) {
    digits[i] = num%10;
    num /= 10;
  }  

  // Fuehrende Nullen unterdruecken
  for (int8_t i = N_DIGITS-1; i>0; i--) {
    if (digits[i] == 0) digits[i] = 16;
    else break;
  }
  
  // An LED ausgeben
  for (int8_t i = N_DIGITS-1; i>=0; i--) {
     sgl_digits[i] = led_lut0[digits[i]];
  }  
}


void led_mpx_init(void) {
    // LEDs und Multiplex beide active low
    LED_DDR = 0xFF;
    GRP_DDR  |= ((1<<GRP_0) | (1<<GRP_1) | (1<<GRP_2));
    GRP_PORT |= ((1<<GRP_0) | (1<<GRP_1) | (1<<GRP_2)); 
}

void led_mpx_deinit(void) {
  // Alles aus zum stromsparen
  // LEDs und Multiplex beide active low 
  LED_DDR  = 0x00;
  LED_PORT = 0xFF;
  GRP_DDR  &= ~((1<<GRP_0) | (1<<GRP_1) | (1<<GRP_2));
  GRP_PORT |= ((1<<GRP_0) | (1<<GRP_1) | (1<<GRP_2));
}




// Blinken ein- und ausschalten
void led_mpx_blink(uint8_t y) {
  if(y) sgl_isblink = 1;
  else  sgl_isblink = 0;
}

// Beliebige Bitmuster auf LED darstellen
void led_mpx_putany(char *s) {
  for (uint8_t i=0; i<N_DIGITS; i++) {
    sgl_digits[i] = s[i];
  }
}