/*
* clock.c
*
* Kleine Steuerung fuer einen Modellbauservo
* benoetigt Timer 1
*
* Created: 17.08.2013
*  Author: Nicolas (www.DL1DOW.de)
*/

#include <avr/io.h>
#include <stdbool.h>
#include "ioMapping.h"
#include "servo.h"
#include "led_multiplex.h"


// Vorzeichen
static int8_t sign(int16_t a) {
    if (a>0) return 1;
    else if (a<0) return -1;
    else return 0;
}

// Servo: Pins und Timer vorbereiten
void servo_init(uint16_t startpos)
{
    SERVO_PWR_PORT &= ~(1<<SERVO_PWR); // active low
    SERVO_PWR_DDR  |=  (1<<SERVO_PWR);
    SERVO_SIG_DDR  |=  (1<<SERVO_SIG);
    
    // DB S. 132
    // PWM mode 14: Fast PWM, TOP = ICR1, BOTTOM = 0
    // Prescaler: 8
    TCCR1A = (1<<COM1A1) | (0<<COM1A0) | (1<<WGM11) | (0<<WGM10); // clear on compare match
    TCCR1B = (0<<CS12) | (1<<CS11) | (0<<CS10) | (1<<WGM13) | (1<<WGM12);
    ICR1 = F_CPU/8/50;  // fix, ca. 20ms, 16-bit-Register

    // startpos = 0 ist ein ungueltiger Wert (Timing!), alter Wert wird dann beibehalten
    if (startpos) OCR1A = startpos; // 8-bit-Register, Startwert
}

void servo_off(void) 
{
    SERVO_PWR_PORT |=  (1<<SERVO_PWR); // active low
    SERVO_PWR_DDR  &= ~(1<<SERVO_PWR);
    SERVO_SIG_DDR  &= ~(1<<SERVO_SIG);
}

void servo_on(void) 
{
    SERVO_PWR_DDR  |=  (1<<SERVO_PWR);
    SERVO_PWR_PORT &= ~(1<<SERVO_PWR);
    SERVO_SIG_DDR  |=  (1<<SERVO_SIG);
}


// Servo einen Schritt pro Aufruf in die richtige Richtung bewegen
bool servo_move(uint16_t pos) 
{
    uint16_t lastpos = OCR1A;
    uint16_t aktpos = pos;
    if (pos==lastpos) {
        return true;
    }

    #ifdef DEBUG
        led_mpx_putnum(OCR1A);
    #endif

    SERVO_PWR_PORT &= ~(1<<SERVO_PWR); // active low
    OCR1A += (sign((int16_t) aktpos- (int16_t) lastpos));
    
    return false; 
}